class ResponseCode:
    SUCCESS = 8000
    INVALID_REQUEST = 8001
    NO_RECORD_FOUND = 8002
    UNAUTHORIZED = 8003
    DUPLICATE = 8004
    FAILURE = 8005
    DATA_IN_USE = 8006
    BAD_REQUEST = 8007
    METHOD_NOT_ALLOWED = 8008
    RESTRICTED_ACCESS = 8009
    SESSION_EXPIRED = 8010
    TOKEN_EXPIRED = 8050
    NULL_ARGUMENT = 8011
    NO_DATA_CHANGED = 8012
    FILE_NOT_EXIST = 8013
    NO_PARENT_RECORD = 8016
    PROCESS_ALREADY_COMPLETED = 8021
    IS_NOT_FOR_SUBMISSION = 8022
    PROCESS_NOT_DEFINED = 8023
    NO_APPROVAL_ACTION = 8024
    NO_FINANCIAL_YEAR = 8025
    SUBMISSION_STAGE_NOT_DEFINED = 8026
    RECEIVING_STAGE_NOT_DEFINED = 8027
    ALREADY_SUBMITTED = 8028
    RECORD_IS_DELETED = 8029
    RECORD_IS_DROPPED = 8030
    RECORD_IS_IN_PROGRESS = 8031
    RECORD_IS_IN_APPROVAL_PROCESS = 8035
    ALREADY_APPROVED = 8037
    NOT_INITIALISED = 8038
    PARTIAL_SUCCESS = 8040
    ALREADY_ASSIGNED = 8042
    DUPLICATE_ASSIGNMENT = 8044
    LIMIT_REACHED = 8045